@@ -11,21 +11,19 @@ reorderGroups <- function(group, m, bilevel) {
     ord <- order(g)
     ord.inv <- match(1:length(g), ord)
     g <- g[ord]
+    gLevels <- setdiff(levels(gf), "0")
   } else {
     reorder <- FALSE
     g <- group
     ord <- ord.inv <- NULL
+    gLevels <- paste0("G", unique(g[g!=0]))
   }
-  J <- max(g)
+  J <- length(gLevels)
   if (missing(m)) {
     m <- if (bilevel) rep(1, J) else sqrt(table(g[g!=0]))
   }
-  if (length(m)!=max(g)) stop("Length of group.multiplier must equal number of penalized groups")
-  if (reorder) {
-    names(m) <- setdiff(levels(gf), "0")
-  } else {
-    names(m) <- paste0("G", unique(g[g!=0]))
-  }
+  names(m) <- gLevels
+  if (length(m) != J) stop("Length of group.multiplier must equal number of penalized groups")
   if (storage.mode(m) != "double") storage.mode(m) <- "double"
   list(g=g, m=m, ord=ord, ord.inv=ord.inv, reorder=reorder)
 }
